var googleChartsInited = false

function initChart() {
	if (googleChartsInited == false) {
		googleChartsInited=true;
		google.charts.load('current', {
			'packages' : [ 'corechart' ]
		});
	}
}

AJS.$(document).ready(function() {
	findNewChart();
	JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
		if (reason == 'dialogContentReady' || reason == 'dialogReady') {
			findNewChart();
		}
	});
});

function findNewChart() {
	initChart();
	AJS.$("[name=vm_chart]").each(function() {
		var obj = AJS.$(this);
		if (obj.attr("inited")=="false") {
			obj.attr("inited", "true");
			var newChart = new VacationManagerChart(obj);
			newChart.init();
		}
	});
}

function VacationManagerChart(_container) {
	var container = _container;
	var chartId = container.attr("chart-id");
	var formBody = container.closest(".form-body");
	var fieldGroup = container.closest(".field-group");
	
	var vacationDaysChart;
	var vacationDaysChartData;
	var startDateCf;
	var endDateCf;
	var chartCfId;
	var reporterField;
	var vacationKindId;
	var issueId;
	var reporter;
	
	var validation = "";
	var errors = null;

	var availableDays = 0;
	var entitledDays = 0;
	var leftoverDays = 0;
	var usedDays = 0;
	var reservedDays = 0;
	var usedByThisIssueDays = 0;
	var selectedDays = 0;

	var hasDayLimit;
	
	var periodStart;
	var periodEnd;
	
	var vmMessages;

	var options = {
		chartArea:{left:10,top:10,width:'50%',height:180},
		legend: {textStyle: {color: '#707070'}},
		colors : [
		           '#67ab49', //available/entitled days
		           '#59afe1',  //chosen days
		           '#f79232', //reserved days
		           '#d04437' //used days
	           ],
		pieSliceText : 'value'
	};
	
	function init() {
		initMsgContainers();
		initValuesFromDocument();
		
		google.charts.setOnLoadCallback(function() {
			vacationDaysChart = new google.visualization.PieChart(container.find("[name=chart_div]")[0]);
			updateChart();
		});

		startDateCf.on('change', function() {
			var startDate = startDateCf.val();
			var endDate = endDateCf.val();
			if (!endDate) {
				endDateCf.val(startDate);
			}
			onFieldsChanged();
		});
		endDateCf.on('change', function() {
			var startDate = startDateCf.val();
			var endDate = endDateCf.val();
			if (!startDate) {
				startDateCf.val(endDate);
			}
			onFieldsChanged();
		});
		reporterField.on('change', function() {
			onFieldsChanged();
		});
		onFieldsChanged();
	}
	
	function initMsgContainers() {
		
		var newFieldGroup = $("<div class='field-group'></div>");
		vmMessages = $("<div class='error'></div>");
		
		newFieldGroup.append(vmMessages);		
		fieldGroup.after(newFieldGroup);

        var hintText = AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.send.to.supervisor");
        formBody.next().append('<div id="vm-info" class="aui-dialog2-footer-hint" style="text-align:left;">'+hintText+'</div>');
	}

	function updateChart() {

		if (errors!=null) {
			fieldGroup.hide();
			var message = "";
			for (var i=0; i<errors.length; i++) {
				message+="<div>"+errors[i].message+"</div>";
			}
			showWarningMessage(message);
			return;
		}
		
		availableDays=entitledDays+leftoverDays-usedDays-selectedDays-reservedDays;
		if (availableDays<0)
			availableDays=0;

		if (hasDayLimit == false)
			entitledDays = 0;

		if (availableDays<=0 && reservedDays<=0 && usedDays<=0 && selectedDays<=0) {
			if (hasDayLimit==true) {
				validation = AJS.I18n.getText("vacationmanager.vacation.validator.no.available.days.in.period");
			}
			fieldGroup.hide();
		} else {
			vacationDaysChartData = new google.visualization.DataTable();
			vacationDaysChartData.addColumn('string', 'Days');
			vacationDaysChartData.addColumn('number', 'Amount');
			vacationDaysChartData.addColumn({
				type : 'string',
				role : 'tooltip'
			});
			
			var availableDaysLabel = AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.available.days.description", availableDays);
			var leftoverDaysLabel = (leftoverDays>0)?" "+AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.leftover.days.description", leftoverDays):"";
			
			dataTable = [
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.available.days"), availableDays, availableDaysLabel + leftoverDaysLabel ],
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.chosen.days"), selectedDays,
				  	AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.chosen.days.description", selectedDays) ],
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.reserved.days"), reservedDays,
				  	AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.reserved.days.description", reservedDays) ],
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.used.days"), usedDays,
				  	AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.used.days.description", usedDays) ]
			];

			if (periodStart!="" && periodEnd!="") {
				var periodTypeInfo="";
				var now=new Date();
				now.setHours(0);
				now.setMinutes(0);
				now.setSeconds(0);
				now.setMilliseconds(0);
				if (periodEnd<periodStart) {
					periodTypeInfo = " <span style='color:#d04437;'>("+AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.invalid")+")</span>";
				} else if (periodStart>now) {
					periodTypeInfo = " <span style='color:#14892c;'>("+AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.future")+")</span>";
				} else if (periodEnd<now) {
					periodTypeInfo = " <span style='color:#f6c342;'>("+AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period.past")+")</span>";
				}
				container.find(".periodInfo").html(AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.period", VM$.formatDate(periodStart), VM$.formatDate(periodEnd))+periodTypeInfo);
				container.find(".periodInfo").show();
			} else {
				container.find(".periodInfo").hide();
			}
			
			if (reporter!=null) {
				container.find(".reporterInfo").text(AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.reporter", reporter));
				container.find(".reporterInfo").show();
			} else {
				container.find(".reporterInfo").hide();
			}
			
			vacationDaysChartData.addRows(dataTable);
			
			fieldGroup.show();
			vacationDaysChart.draw(vacationDaysChartData, options);
		}

		showWarningMessage(validation);
	}

	function initValuesFromDocument() {
		var startDateCfId=container.find("[name=chart_startDateCfId]").val();
		var endDateCfId=container.find("[name=chart_endDateCfId]").val();
		startDateCf=formBody.find('[name=customfield_' + startDateCfId+"]");
		endDateCf=formBody.find('[name=customfield_' + endDateCfId+"]");
		reporterField=formBody.find("#reporter");
		
		chartCfId=container.find("[name=chart_chartCfId]").val();
		
		selectedDays = parseFloat(container.find("[name=chart_selectedDays]").val());
		if (!selectedDays) {
			selectedDays = 0;
		}
		entitledDays=parseFloat(container.find("[name=chart_entitledDays]").val());
		leftoverDays=parseFloat(container.find("[name=chart_leftoverDays]").val());
		usedDays=parseFloat(container.find("[name=chart_usedDays]").val());
		reservedDays=parseFloat(container.find("[name=chart_reservedDays]").val());
		usedByThisIssueDays=parseFloat(container.find("[name=chart_daysUsedByThisIssue]").val());

		hasDayLimit=(container.find("[name=chart_hasDayLimit]").val()=="true");
		
		vacationKindId=container.find("[name=chart_vacationKindId]").val();

		periodStart = new Date(container.find("[name=chart_periodStart]").val());
		periodEnd = new Date(container.find("[name=chart_periodEnd]").val());
		
		issueId = container.find("[name=chart_issue_id]").val();
		
		validation = container.find("[name=chart_validation]").val();
		
		reporter=null;
	}
	
	function parseValuesFromJson(json) {
		selectedDays = parseFloat(json.selectedDays);
		
		entitledDays = parseFloat(json.entitledDays);
		leftoverDays = parseFloat(json.leftoverDays);
		usedDays = parseFloat(json.usedDays);
		reservedDays = parseFloat(json.reservedDays);
		usedByThisIssueDays = parseFloat(json.usedByThisIssueDays);
		
		periodStart = new Date(json.periodStart);
		periodEnd = new Date(json.periodEnd);
		
		validation = json.validation;
		
		if (json.reporter!=undefined && json.reporter.length>0) {
			reporter=json.reporter;
		} else {
			reporter=null;
		}
	}

	function onFieldsChanged() {
		var endDate = endDateCf.val();
		var startDate = startDateCf.val();
		var reporter = reporterField.val();
		if (reporter!=null && reporter!=undefined && reporter.constructor === Array) {
			if (reporter.length>0) {
				reporter=reporter[0];
			} else {
				reporter="";
			}
		}
		if (!(!startDate || !endDate)) { // check one of the dates is blank
			getVacationInfo(startDate, endDate, reporter);
		}
	}
	
	function getVacationInfo(startDate, endDate, reporter) {
		container.find(".vm_chart_spin").spin();
		var params = new Object();
		params.vacationKindId = vacationKindId;
		params.selectedStartDate = startDate;
		params.selectedEndDate = endDate;
		if (reporter!=null && reporter!=undefined) {
			params.reporter=reporter;
		}
		if (issueId!=undefined && issueId!="") {
			params.issueId = issueId;
		}

		jQuery
				.ajax(
						{
							type : "POST",
							url : AJS.contextPath() + "/rest/vacation-manager/1.0/user-vacation/vacation-info",
							data : JSON.stringify(params),
							dataType : "json",
							contentType: 'application/json; charset=utf-8',
							async : true
						}).done(function(data) {
					container.find(".vm_chart_spin").spinStop();
					if (data.errors==undefined) {
						parseValuesFromJson(data);
						errors = null;
					} else {
						errors = data.errors;
					}
					updateChart();
				});
	}

	function showWarningMessage(text) {
		if (text==undefined || text==null || text.length==0) {
			hideWarningMessage();
			return;
		}

		vmMessages.text(text);
		vmMessages.show();
	}

	function hideWarningMessage() {
		vmMessages.hide();
	}

	return {
		init : init
	};
}
