AJS.$(document).ready(function() {

    VM$.configuration.getShowUserPickerVacationWarning(function(doShow) {
        if (doShow == true) {
            var userVacationInfo = new UserVacationInfo();

            findUserPickerFields(userVacationInfo);
            findUserFields(userVacationInfo);
            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                if (reason == 'dialogContentReady' || reason == 'dialogReady') {
                    findUserPickerFields(userVacationInfo);
                    findUserFields(userVacationInfo);
                }
            });

            JIRA.bind(JIRA.Events.PANEL_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });

            JIRA.bind(JIRA.Events.ISSUE_REFRESHED, function(e, context, reason) {
                findUserPickerFields(userVacationInfo);
                findUserFields(userVacationInfo);
            });
        }
    });
});

function UserVacationInfo() {
    var ajaxCacher = new AjaxCacher(AJS.$);
    
    function getCurrentVacationInfoForUser(userName, responseCallback) {
        var params = new Object();
        params.userName = userName;
        
        ajaxCacher.ajax({
            type : "POST",
            url : AJS.contextPath() + "/rest/vacation-manager/1.0/user-vacation/is-on-vacation",
            data : JSON.stringify(params),
            dataType : "json",
            contentType : 'application/json; charset=utf-8',
            async : true,
            cacheId: userName
        }, responseCallback);
    }

    return {
        getCurrentVacationInfoForUser : getCurrentVacationInfoForUser
    }
}

function findUserPickerFields(userVacationInfo) {
    AJS.$("input.userpickerfield,select.single-user-picker").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserPickerWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user picker field: " + ex);
            }
        }
    });
}

function UserPickerWarning(field, userVacationInfo) {
    var messageContainer = null;

    function initMessageContainer() {
        messageContainer = $("<div class='vm-warning-field-group'></div>");
        field.closest("div.field-group").append(messageContainer);
        messageContainer.hide();
    }

    field.on('change', function() {
        checkUser(getUserName());
    });

    function checkUser(userName) {
        userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
            if (data != null && data.hasVacation == true) {
                showWarningMessage(AJS.I18n.getText("vacationmanager.userpicker.warning.has.vacation.until", VM$
                        .formatDate(new Date(data.hasVacationUntil))));
            } else {
                showWarningMessage(null);
            }
        });
    }

    function showWarningMessage(text) {
        if (text == undefined || text == null || text.length == 0) {
            messageContainer.hide();
            return;
        }

        messageContainer.text(text);
        messageContainer.show();
    }

    function getUserName() {
        return field.val().toString();
    }

    initMessageContainer();
    checkUser(getUserName());
}

function findUserFields(userVacationInfo) {
    AJS.$("[data-user][rel],[data-fieldtype='userpicker']").each(function() {
        var obj = AJS.$(this);
        if (obj.attr("vm-inited") != "true") {
            obj.attr("vm-inited", "true");
            try {
                new UserFieldWarning(obj, userVacationInfo);
            } catch (ex) {
                console.log("Cannot parse user field: " + ex);
            }
        }
    });
}

function UserFieldWarning(field, userVacationInfo) {
    var userName = field.attr("rel");
    if (userName == undefined) {
        userName = field.find("[rel]").attr("rel");
    }

    userVacationInfo.getCurrentVacationInfoForUser(userName, function(data) {
        if (data != null && data.hasVacation == true) {
            addWarningMessage(AJS.I18n.getText("vacationmanager.userpicker.warning.has.vacation.until", VM$
                    .formatDate(new Date(data.hasVacationUntil))));
        }
    });

    function addWarningMessage(text) {
        if (text == undefined || text == null || text.length == 0) {
            return;
        }

        var messageContainer = $("<div class='vm-warning-field-group'></div>");
        messageContainer.text(text);
        field.append(messageContainer);
    }
}
